#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import random
import hashlib
import importlib
import sys
import os
import subprocess
import json

from rich.console import Console
from rich.table import Table

console = Console()

from wpy.base import BaseFactory
from wpy.tools import randoms

from wush.common import constants

class FunctionFactory(BaseFactory):
    pass

@FunctionFactory.register()
def random_int(length, min_int=None, max_int=None):
    """随机 int 值"""
    return randoms.random_int(length, min_int, max_int)

RANDOM_STR = ()

@FunctionFactory.register()
def random_str(length, source=None):
    """随机 int 值"""
    return randoms.random_str(length, source)

@FunctionFactory.register()
def md5(text):
    """计算 md5 摘要"""
    return hashlib.md5(text.encode('utf-8')).hexdigest()

@FunctionFactory.register()
def get_current_space_name():
    """获取当前 space 名称"""
    return constants.DEFAULT_SPACE_NAME

@FunctionFactory.register()
def get_completion_words(word_for_completion):
    """获取补全的单词列表"""
    return []

@FunctionFactory.register()
def request(wapi, module_name, request_name):
    return wapi.request(request_name = request_name, module_name = module_name)

@FunctionFactory.register()
def get_current_web_port():
    return os.getenv('WUSH_WEB_PORT')

@FunctionFactory.register()
def handler_response(request, response):
    #  print('Status: {}'.format(response.status_code))
    print('Response:')
    try:
        data = response.json()
        data = json.dumps(data, indent=4, ensure_ascii=False)
    except:
        self.logger.error(traceback.format_exc())
        data = response.content
    print(data)

@FunctionFactory.register()
def print_table(config):
    headers = config.get("headers") or []
    table = Table(show_header=True, show_lines=True, header_style="bold magenta")
    for header in headers:
        table.add_column(header.get("display", '未命名'), )
    items = config.get("items", [])
    for item in items:
        table.add_row(*item)
    console.print(table)
    after_table = config.get("after_table")
    if after_table:
        console.print(after_table)

def run_shell(command):
    """运行 shell 语句"""
    res = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
        stderr = subprocess.PIPE)
    return res.communicate()

def open_version(version, otype=None):
    """使用浏览器打开 url"""
    request_url = ("http://0.0.0.0:{port}/api/version/{version}").format(
        port = get_current_web_port(),
        version = version)
    if otype:
        request_url += '/' + otype
    os.system('open -a "/Applications/Google Chrome.app" "{}"'.format(
        request_url))

def load_module(module_name):
    """加载模块"""
    views_module = importlib.import_module(module_name)
    return views_module

class Function:
    get_current_space_name = None
    get_completion_words = None
    random_int = None
    random_str = None
    get_current_web_port = None
    handler_response = None
    test = None

    _functions = {}

    def __init__(self, functions):
        self._functions = functions
        for name, func in functions.items():
            if isinstance(func, str):
                raise Exception('func {} can not be str'.format(name))
            setattr(self, name, func)

    def add_function(self, func):
        setattr(self, func.__name__, func)
        self._functions[func.__name__] = func

    def get_functions(self):
        return self._functions

_function = Function(FunctionFactory.get_factory())

def get_super_function():
    return _function

super_function = get_super_function()

if __name__ == "__main__":
    print(random_int(5, 4, 9))
    print(random_str(5))
    func = get_super_function()
    print(func.random_int(2))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
run 命令的参数解析
"""
import os
from wpy.argument import Action

from wush.common import utils
from wush.common.functions import super_function
from wush.common.functions import open_version
from wush.common.loggers import create_logger
from .command import CmdArgumentParser
from wush.cli.server import PORT
from wpy.argument import CommandArgumentParserFactory

@CommandArgumentParserFactory.register()
class RunArgumentParser(CmdArgumentParser):
    cmd = 'run'
    logger = create_logger('RunArgumentParser')

    @classmethod
    def default(cls):
        """
        初始化一个实例
        """
        item = cls()
        item.add_argument('cmd')
        item.add_argument('--module', help='模块名称')
        item.add_argument('--space', help='空间名称')
        item.add_argument('--name', help='请求名称')
        item.add_argument('--params', action = Action.APPEND.value,
            help='请求地址参数')
        item.add_argument('--json', action = Action.APPEND.value,
            help='请求 body 参数，json 格式')
        item.add_argument('--open', action = Action.STORE_TRUE.value,
            help = '是否通过浏览器打开请求结果')
        return item

    def get_completions_after_argument(self, wapi, word_for_completion):
        """
        获取补全的单词列表
        :param wapi: Wapi
        :param word_for_completion: 补全需要的单词
        """
        words = []
        if not self.argument:
            return words
        arg = self.argument
        try:
            wapi.init_config(
                module_name = arg.module,
                request_name = arg.name
            )
        except:
            pass
        if word_for_completion == '--name':
            module_name = wapi.module_name
            requests = wapi.config.get_requests(module_name)
            words = []
            for req in requests:
                words.append(dict( text = req.get("name"),
                    display_meta=req.get("title")
                    ))
            return words
        elif word_for_completion == '--module':
            modules = wapi.config.get_modules()
            words = [ dict(text = o) for o in modules ]
            return words
        elif word_for_completion == '--params':
            # 参数补全
            module_name = wapi.module_name
            request = wapi.config.get_module(module_name
                ).get_request(wapi.request_name)
            params = request.params or {}
            words = self._dict_to_completions(params)
            return words
        elif word_for_completion == '--json':
            # 参数补全
            module_name = wapi.module_name
            request = wapi.config.get_module(module_name
                ).get_request(wapi.request_name)
            params = request.json or {}
            words = self._dict_to_completions(params)
            return words

        return super().get_completions_after_argument(wapi, word_for_completion)

    def _dict_to_completions(self, data):
        words = []
        for k, v in data.items():
            words.append(dict(
                text = '{}='.format(k),
                display = '{}={}'.format(k, v),
                display_meta=''))
        return words

    def run(self, text):
        args = self.parse_args(text)
        if not args.name:
            raise Exception
        _params = args.params or []
        params = utils.list_key_val_to_dict(_params)
        json_data = utils.list_key_val_to_dict(args.json or [])

        self.wapi.build(
            space_name = args.space,
            module_name = args.module,
            request_name = args.name,
            params = params,
            json = json_data
        )

        self.logger.info('arg params %s', params)

        self._print('Space: {}'.format(self.wapi.space_name))
        self._print('Module: {}'.format(self.wapi.module_name))
        self._print('Request: {}'.format(self.wapi.request_name))
        self._print('Url: {}'.format(self.wapi.url))
        self._print('Params: {}'.format(self.wapi._request_data.get("params")))
        self._print('Json: {}'.format(self.wapi._request_data.get("json")))
        self._print('请求中。。。')

        self.wapi.request()

        self._print('Status: {}'.format(self.wapi.response.status_code))
        #  self._print('Response:')

        self.wapi.save()
        if args.open:
            self._print('See in browser')
            self._open()
        else:
            #  self._print(self.wapi.get_pertty_response_content())
            self.wapi.config.get_function().handler_response(
                self.wapi._request, self.wapi.response)

    def _open(self):
        #  """打开请求信息"""
        open_version(self.wapi.version, 'response')



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
工具模块
"""

import json
import os

from collections import defaultdict

from wush.common.functions import run_shell
from wush.common.exceptions import JsonException

def filter_json(data, rules):
    """
    过滤 json 数据
    :param dict data: 过滤的数据
    :param list rules: 过滤规则，例：{"name"}

    最终拼成的命令格式如下
    echo '{"id": 1, "name": "wxnacy"}' | jq '{"name"}'
    """
    cmd_fmt = "echo '{}' | jq '{}'"
    text = json.dumps(data)
    for rule in rules:
        cmd = cmd_fmt.format(text , rule)
        content, err = run_shell(cmd)
        if err:
            raise JsonException(err)
        return json.loads(content)
    return data

def fmt_path(path):
    """格式化地址"""
    path = os.path.expanduser(path)
    return path

def search(datas, word):
    """
    搜索
    :param list datas: 单词列表
    :param str word: 搜索的单词
    :results list 搜索结果，按照匹配位置排序
    """
    patten = defaultdict(int)
    for o in datas:
        patten[o] = -1
        p = o.lower()
        if word not in p:
            continue
        patten[o] = p.index(word)
    res = list(filter(lambda x: patten[x] > -1, datas))
    res.sort(key = lambda x: patten[x])
    return res

def list_key_val_to_dict(data):
    """
    键值对列表转为字典
    :param data: ['name=wxnacy', 'key=True', 'test']
    :returns { 'name': 'wxnacy', 'key': 'True'  }
    """
    res = {}
    for item in data:
        if '=' in item:
            k, v = item.split('=')
            res[k] = v
    return res




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
models
"""
import json
import sys
import os
import requests

from lfsdb import FSModel
from lfsdb import FSColumn

from wush.common import constants
from wush.common.config_value import ConfigValue
from wush.common.cookie import Cookie
from wush.common.files import FileUtils
from wush.common.loggers import create_logger

class BaseModel(object):

    @classmethod
    def _format_str(cls, value):
        if not isinstance(value, str):
            return value

        if value.startswith('@'):
            with open(value[1:], 'r') as f:
                data = []
                for line in f:
                    data.append(line.strip('\n'))
                content = ''.join(data)
                if content.startswith('{') and content.endswith('}'):
                    return json.loads(content)
                return content
        return value

    @classmethod
    def _set_item_attr(cls, item, k, v):
        if hasattr(item, k):
            value = ConfigValue(v).set_env(**item.env).set_functions(
                    **item.functions).format()
            setattr(item, k, value)

    def __str__(self):
        return str(self.__dict__)

    def pretty_str(self):
        data = self.dict()
        functions = data.get("functions")
        if functions:
            for k, v in functions.items():
                functions[k] = k
        return json.dumps(data, indent=4)

    def dict(self):
        data = dict(self.__dict__)
        data.pop('_config', None)
        data.pop('logger', None)
        return data

    def format(self):
        for k, v in self._config.items():
            self._set_item_attr(self, k, v)

    def add_attr(self, k, v):
        """增加属性"""
        if not v:
            return

        if not hasattr(self, k):
            setattr(self, k, v)
            return

        if isinstance(v, dict):
            val = getattr(self, k) or {}
            val.update(v)
            setattr(self, k, val)
            return

        setattr(self, k, v)

    def add_attrs(self, **kwargs):
        for k, v in kwargs.items():
            self.add_attr(k, v)

class ModuleModel(BaseModel):
    logger = create_logger('ModuleModel')
    parent = ''
    module = ''
    protocol = 'http'
    cookie_domains = []
    cookies = {}
    headers = {}
    env = {}
    url_prefix = ''
    requests = []
    _config = {}

    @classmethod
    def load(cls, config):
        item = cls()
        item._config = config
        item.add_attrs(**config)
        item._config.pop('requests', None)
        item.format()
        return item

    @classmethod
    def _merge_config(cls, parent_config, sub_config):
        """合并配置"""
        for k, v in sub_config.items():
            if isinstance(v, dict):
                pv = parent_config.get(k) or {}
                pv.update(v)
                parent_config[k] = pv
            else:
                parent_config[k] = v
        return parent_config

    def format(self):
        for k, v in self._config.items():
            self._set_item_attr(self, k, v)

        # 获取浏览器的 cookies
        if self.cookie_domains:
            total_cookies = self._config.get("cookies") or {}
            for domain in self.cookie_domains:
                _cookies = Cookie.get_cookie(domain) or {}
                total_cookies.update(_cookies)
            self._config['cookies'] = total_cookies

    def get_request(self, name, **kwargs):
        for item in self.requests:
            if item.get("name") == name:
                item = self._merge_config(dict(self._config), item)

                req_model = RequestModel.load(item)
                # 加载参数中的变量
                for k in ('env', 'params', 'json', 'data', 'headers', 'cookies'):
                    v = kwargs.get(k)
                    req_model.add_attr(k, v)
                return req_model

class RequestModel(BaseModel):
    logger = create_logger('RequestModel')
    _config = {}
    module = ''
    domain = ''
    url_prefix = ''
    cookies = {}
    headers = {}
    env = {}
    functions = {}

    protocol = 'http'
    name = ''
    title = ''
    path = ''
    method = 'get'
    data = ''
    json = {}
    params = {}
    filters = []

    url = ''

    @classmethod
    def load(cls, config):
        item = cls()
        item._config = config
        cls.logger.info('Total cookies: %s', config.get("cookies"))
        item.add_attrs(**config)
        item.format()
        return item

    def format(self):
        for k, v in self._config.items():
            self._set_item_attr(self, k, v)
        if not self.title:
            self.title = self.name
        if not self.domain:
            self.domain = '127.0.0.1'

        self.method = self.method.upper()
        # 如果不是完整地址，拼接出完整地址
        self.url = self.path
        if not self.path.startswith('http'):
            self.url = self.url_prefix + self.path
            self.url = '{protocol}://{domain}{url}'.format(
                protocol = self.protocol,
                domain = self.domain, url = self.url)
        self.logger.info('Url: %s', self.url)

        # 处理 headers
        method_headers = self.headers.pop(self.method, None
            ) or self.headers.pop(self.method.lower(), None)
        remove_method_keys = []
        for k, v in self.headers.items():
            if k.upper() in constants.METHODS and isinstance(v, dict):
                remove_method_keys.append(k)
        for key in remove_method_keys:
            self.headers.pop(key, None)

        if isinstance(method_headers, dict):
            self.headers.update(method_headers)

class Client(FSModel):
    db = 'wush'
    table = 'client'

    port = FSColumn(int,)

class Version(FSModel):
    db = 'wush'
    table = 'version'

    version = FSColumn(str,)
    space_name = FSColumn(str,)
    module_name = FSColumn(str,)
    request_name = FSColumn(str,)
    config = FSColumn(dict,)

class Request(FSModel):
    db = 'wush'
    table = 'request'

    #  version = FSColumn(str,)
    #  space_name = FSColumn(str,)
    #  module_name = FSColumn(str,)
    #  request_name = FSColumn(str,)
    #  config = FSColumn(dict,)

class Response(FSModel):
    db = 'wush'
    table = 'response'

    content = FSColumn(str,)
    json = FSColumn(dict,)
    headers = FSColumn(dict,)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

import sys
import json

from wush.common.files import FileUtils

chrome = None
try:
    import browsercookie
    chrome = browsercookie.chrome()
except :
    print('can not import browsercookie')
    pass

class Cookie():

    @classmethod
    def get_cookie(cls, domain):
        res = {}
        if not chrome:
            return res
        try:
            for k, v in chrome._cookies.items():
                if domain == k:
                    cookies = v.get("/")
                    for ck, cv in cookies.items():
                        res[ck] = cv.value
        except Exception as e:
            print(e)
        return res

    @classmethod
    def get_cookie_and_save(cls, domain, filepath):
        file_data = FileUtils.read_dict(filepath)
        res = cls.get_cookie(domain)
        if res:
            with open(filepath, 'w') as wf:
                file_data[domain] = res
                wf.write(json.dumps(file_data, indent=4))
        return file_data.get(domain, {})

    @classmethod
    def get_cookie_from_path(cls, domain, filepath):
        file_data = FileUtils.read_dict(filepath)
        return file_data.get(domain, {})




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

import os

CONFIG_USER_ROOT = os.path.join(os.getenv("HOME"), '.wapi')
CONFIG_USER_PATH = os.path.join(CONFIG_USER_ROOT, 'wapi.yml')

CONFIG_ROOT = CONFIG_USER_ROOT
CONFIG_PATH = os.path.join(CONFIG_ROOT, 'wapi.yml')
ENV_PATH = os.path.join(CONFIG_ROOT, 'env.yml')
REQUEST_PATH = os.path.join(CONFIG_ROOT, 'request.yml')

DEFAULT_SPACE_NAME = 'default'
DEFAULT_MODULE_NAME = 'default'

FUNC_NAME_GET_CURRENT_SPACE_NAME = 'get_current_space_name'

DEFAULT_NAME_MODULE_ROOT = 'module_root'

# 默认配置
DEFAULT_CONFIG = {
    'module_root': 'module',
    'env_root': 'env',
    'body_root': 'body',
    'data_root': '~/.wapi/data',
    'function_moduls': []
}

# request 请求方式
METHODS = ('GET', 'OPTIONS', 'HEAD', 'POST', 'PUT', 'PATCH', 'DELETE')




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
run 命令的参数解析
"""

from .command import CmdArgumentParser
from wpy.argument import CommandArgumentParserFactory

@CommandArgumentParserFactory.register()
class ConfigArgumentParser(CmdArgumentParser):
    cmd = 'config'

    @classmethod
    def default(cls):
        """
        初始化一个实例
        """
        item = cls()
        item.add_argument('cmd')
        item.add_argument('--config')
        item.add_argument('--module')
        item.add_argument('--space')
        return item

    def run(self, text):
        args = self.parse_args(text)
        if args.has_args():
            self.wapi.init_config(
                space_name = args.space,
                module_name = args.module,
                config_root = args.config)
        else:
            self._print('root={}'.format(self.wapi.config_root))
            self._print('module={}'.format(self.wapi.module_name))
            self._print('space={}'.format(self.wapi.space_name))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import abc
import pygments

from pygments.token import Token
from pygments.lexers.python import PythonLexer
from prompt_toolkit.formatted_text import PygmentsTokens
from prompt_toolkit import print_formatted_text

from wpy.argument import CommandArgumentParser

from wush.common.loggers import create_logger

class CmdArgumentParser(CommandArgumentParser):
    logger = create_logger('CmdArgumentParser')
    wapi = None
    prompt_session = None

    def set_wapi(self, client):
        self.wapi = client

    def set_prompt_session(self, client):
        self.prompt_session = client

    def get_completions_after_argument(self, wapi, word_for_completion):
        """
        获取补全的单词列表
        :param wapi: Wapi
        :param word_for_completion: 补全需要的单词
        """
        return []

    def get_completions_after_cmd(self, argument, words=None):
        """获取补全使用的单词列表"""
        res = []
        if words and isinstance(words, list):
            res.extend(words)
        args = self.get_arguments()
        for arg in args:
            if arg.is_cmd:
                continue
            # 已经赋值的不需要展示
            # 列表除外
            if arg.is_set and not arg.is_list:
                continue
            res.append(dict(text = '--' + arg.name, display_meta = arg.help))
        return res

    def _print(self, text):
        tokens = list(pygments.lex(text, lexer=PythonLexer()))
        print_formatted_text(PygmentsTokens(tokens), end='')



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
run 命令的参数解析
"""
import os
from wpy.argument import Action

from wush.common import utils
from wush.common.loggers import create_logger
from .command import CmdArgumentParser
from wush.cli.server import PORT
from wpy.argument import CommandArgumentParserFactory

from rich.console import Console
from rich.table import Table

console = Console()

@CommandArgumentParserFactory.register()
class FuncArgumentParser(CmdArgumentParser):
    cmd = 'func'
    logger = create_logger('FuncArgumentParser')

    @classmethod
    def default(cls):
        """
        初始化一个实例
        """
        item = cls()
        item.add_argument('cmd')
        return item

    def run(self, text):
        functions = self.wapi.config.get_function().get_functions()
        max_func_length = max([len(o) for o in functions.keys()])
        table = Table(show_header=True, show_lines=True, header_style="bold magenta")
        table.add_column("方法名", width=max_func_length)
        table.add_column("描述", )
        table.add_column("模块", )

        for k, v in functions.items():
            table.add_row(k, getattr(v, '__doc__'), getattr(v, '__module__'))
        console.print(table)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

import json
import os
import re
import subprocess
import yaml

from functools import singledispatch

from wush.common.files import FileUtils
from wush.common.loggers import create_logger

class ConfigValue():
    logger = create_logger('ConfigValue')

    def __init__(self, value):
        self.value = value

        self.REG_ENV = r'(\${.*?})'
        self.REG_PARSE = r'(^\b(json|yml|xml)\b@.*?)'
        self.env = dict(os.environ)
        self.functions = {}

    def set_env(self, **data):
        self.env.update(data)
        return self

    def set_functions(self, **data):
        self.functions.update(data)
        return self

    def format(self):
        '''输出格式化信息'''
        if not self.value:
            # 为空，返回
            return self.value

        #  self.logger.info('ConfigValue env %s', self.env)

        #  格式化响应格式数据
        for t in (str, list, dict):
            if isinstance(self.value, t):
                func_name = '_format_' + t.__name__
                return getattr(self, func_name)(self.value)
        return self.value

    @singledispatch
    def _format(self, obj):
        self.logger.info('type %s', type(obj))
        return obj

    @_format.register(dict)
    def _(self, value):
        """格式化字典"""
        return self._format_dict(dict)

    @_format.register(list)
    def _(self, lines):
        """格式化数组"""
        self.logger.info('-' * 200)
        return self._format_list(lines)

    @_format.register(str)
    def _(self, text):
        """格式化字符窜"""
        self.logger.info('-' * 200)
        return self._format_str(text)

    def _format_dict(self, value):
        """格式化字典"""
        for v_k, v_v in value.items():
            value[v_k] = ConfigValue(v_v).set_env(**self.env
                    ).set_functions(**self.functions).format()
        return value

    def _format_list(self, lines):
        """格式化数组"""
        for i in range(len(lines)):
            lines[i] = ConfigValue(lines[i]).set_env(**self.env
                    ).set_functions(**self.functions).format()
        return lines

    def _format_str(self, text):
        '''格式化字符串'''
        if self._has_environs(text):
            text = self._format_environ(text)
        if text.startswith('@'):
            text = text[1:]
            self.logger.debug(text)
            if text.startswith('{') and text.endswith('}'):
                return json.loads(text)
            if text.startswith('[') and text.endswith(']'):
                return json.loads(text)

            if not self._is_file(text):
                raise ValueError(u'{} 文件路径不存在'.format(text))

            return self._format_file_content(text)

        parse_type = self._get_parse_type(text)
        if parse_type:
            text = text[len(parse_type) + 1:]
            if self._is_file(text):
                if parse_type == 'json':
                    return FileUtils.read_dict(text)
            else:
                if parse_type == 'json':
                    return json.loads(text)

        return text

    def _get_parse_type(self, text):
        """获取解析类型"""
        lines = re.findall(self.REG_PARSE, text)
        if not lines:
            return None
        return lines[0][-1]

    def _format_file_content(self, filepath):
        '''格式化文件内容'''
        if filepath.endswith('.json') or filepath.endswith('.yml'):
            return FileUtils.read_dict(filepath)

        with open(filepath, 'r') as f:
            lines = f.readlines()
            if not lines:
                return ''
            if lines[0].startswith('{') and lines[-1].endswith('}'):
                return FileUtils.read_dict(filepath)
            return ''.join(lines)

    def _is_file(self, filepath):
        '''是否为文件'''
        return os.path.exists(filepath)

    def _has_environs(self, text):
        '''是否有环境变量'''
        if not isinstance(text, str):
            return False
        lines = re.findall(self.REG_ENV, text)
        self.environ_names = [o[2:-1] for o in lines]
        return True if self.environ_names else False

    def _format_environ(self, text):
        '''格式化环境变量'''
        if not self.environ_names:
            return text

        self.logger.info('environ_names %s', self.environ_names)

        for k in self.environ_names:
            # 处理函数的执行和替换
            orgl = '${' + k + '}'
            if '(' in k and ')' in k:
                k = k.strip(')').strip(' ')
                func_name, args_str = k.split('(')
                func = self.functions.get(func_name)
                self.logger.info('func_name %s %s', func_name, func)

                if not func:
                    continue
                if not args_str:
                    args_str = '()'
                else:
                    args_str = '({},)'.format(args_str)
                args = eval(args_str)
                repl = func(*args)
                repl = str(repl)
                text = text.replace(orgl, repl)
            else:
                text = text.replace(orgl, self.env.get(k) or '')
        return text




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

import yaml
import os

from enum import Enum
from wush.common import constants
from wush.common import utils
from wush.common.files import FileUtils
from wush.common.functions import load_module
from wush.common.functions import super_function
from wush.common.functions import Function
from wush.common.functions import FunctionFactory
from wush.common.loggers import create_logger
from wush.common.exceptions import RequestException
from wush.models import ModuleModel
from wush.models import RequestModel

class Env():
    # 参数地址
    body_path = ''

    def __init__(self, **kw):
        self.add(**kw)

    def add(self, **kw):
        """添加环境变量"""
        for k, v in kw.items():
            setattr(self, k, v)
            if k.isupper():
                os.environ[k] = v

    def dict(self):
        data = self.__dict__
        return data

class Config():
    logger = create_logger('Config')

    _root = None
    env_root = ''
    body_root = ''
    module_root = ''
    data_root = ''
    #  response_root = ''
    #  request_root = ''
    #  version_root = ''
    space_name = ''
    function_modules = []
    modules = []
    _config = {}

    function  = None
    _env = Env()

    def __init__(self, ):
        """初始化"""
        self._root = self.get_default_root()
        default_config = self.get_default_config()
        for k, v in default_config.items():
            self._setattr(k, v)

    def _setattr(self, k, v):
        """设置属性"""
        if k == 'env':
            k = '_env'
            v = Env(**v)
        setattr(self, k, v)

    @classmethod
    def load(cls, fileroot):
        """
        加载配置
        :param fileroot: 如果是地址则获取内容，如果是数据则直接加载
        """
        cls.logger.info('Config load path: %s', fileroot)
        item = cls()
        data = {}
        if isinstance(fileroot, dict):
            data = dict(fileroot)
        else:
            item._root = fileroot
            filepath = os.path.join(fileroot, 'wapi.yml')
            with open(filepath, 'r') as f:
                data = yaml.safe_load(f)
        item._config = data

        if data:
            for k, v in data.items():
                item._setattr(k, v)

        # 格式化各个 root 配置
        for _root in ('env_root', 'module_root', 'body_root', 'data_root'):
            item._setattr(_root, item.fmt_path(getattr(item, _root)))

        item._load_functions()

        # 创建保持地址
        #  for _root in ('response_root', 'request_root', 'version_root'):
            #  setattr(item, _root, os.path.join(item.data_root,
                #  _root.replace('_root', '')))
            #  if not os.path.exists(getattr(item, _root)):
                #  os.makedirs(getattr(item, _root))

        return item

    @property
    def env(self):
        """获取 env 信息"""
        return self._env

    def _load_functions(self):
        """加载方法"""
        self.logger.info('Config functions %s', self.function_modules)
        if not self.function_modules:
            self.function = super_function
            return
        for module_name in self.function_modules:
            load_module(module_name)

        functions = FunctionFactory.get_factory()
        self.logger.info('load functions %s', functions)

        f = Function(functions)
        self.function = f

    def get_module(self, module_name):
        """获取模块"""
        _config = {}
        if self.modules:
            for m in self.modules:
                if m.get("module") == module_name:
                    _config = m
        elif self.module_root:
            self.logger.info('Module: %s', module_name)
            _path = self.get_module_path(module_name)
            self.logger.info('Module path: %s', _path)
            if not os.path.exists(_path):
                raise RequestException('can not found request config {}'.format(
                    _path))

            _config = FileUtils.read_dict(_path)
        # 获取 env 信息
        _config['functions'] = self.function.get_functions()

        # 父 env
        parent_env = { "env": self.env.dict() }
        _config = ModuleModel._merge_config(parent_env, _config)

        # 获取父配置
        parent = _config.get("parent")
        if parent:
            parent_path = self.get_module_path(parent)
            parent_config = FileUtils.read_dict(parent_path) or {}
            ModuleModel._merge_config(parent_config, _config)
            _config = parent_config
        module = ModuleModel.load(_config)
        return module

    def set_space_name(self, space_name):
        self.space_name = space_name
        self._reload_env()

    def _reload_env(self):
        """重新加载 env"""
        env_path = self.get_env_path(self.space_name)
        self.logger.info('env_path %s', env_path)
        if os.path.exists(env_path):
            space_env = FileUtils.read_dict(env_path) or {}
            self.env.add(**space_env)

    def get_env_path(self, space_name=None):
        """获取 env 配置地址"""
        if not space_name:
            space_name = self.space_name
        if not space_name:
            space_name = super_function.get_current_space_name()
        path = os.path.join(self.env_root, '{}.yml'.format(space_name))
        self.logger.info('Env path %s', path)
        return path

    def get_module_path(self, module_name=None):
        """获取 request 配置地址"""
        if not module_name:
            module_name = self.get_default_module_name()
        return os.path.join(self.module_root, '{}.yml'.format(module_name))

    def get_body_path(self, body_name):
        """获取 body 配置地址"""
        return os.path.join(self.body_root, body_name)

    def get_modules(self):
        """获取模块名称列表"""
        return list(filter(lambda x: not x.startswith('.'), [
            o.replace('.yml', '') for o in os.listdir(self.module_root)]))

    def get_requests(self, module_name):
        """获取请求名称列表"""
        path = self.get_module_path(module_name)
        data = {}
        with open(path, 'r') as f:
            data = yaml.safe_load(f)

        return data.get("requests") or []

    def get_function(self):
        return self.function

    def dict(self):
        return self._config

    @classmethod
    def get_default_root(cls):
        return constants.CONFIG_ROOT

    def fmt_path(self, path):
        if not path:
            return path
        path = os.path.expanduser(path)
        if os.path.isabs(path):
            return path
        return os.path.join(self._root, path)

    @classmethod
    def get_current_body_name(cls, space_name, module_name, request_name):
        """获取当前 body 文件名称"""
        body_name = '{space}_{module}_{request}'.format(
            space = space_name, module=module_name, request = request_name)
        return body_name

    @classmethod
    def get_current_env_name(self, space_name):
        """获取当前 env 文件名称"""
        return space_name

    @classmethod
    def get_default_module_name(cls):
        """获取默认 module 名称"""
        return constants.DEFAULT_MODULE_NAME

    @classmethod
    def get_default_config(cls):
        """获取默认配置"""
        return constants.DEFAULT_CONFIG



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import json
import os

from flask import Flask
from flask import request

from wush.common.functions import random_int
from wush.common.loggers import create_logger
from wush.wush import Wapi

#  import logging
#  log = logging.getLogger('werkzeug')
#  log.setLevel(logging.ERROR)

app = Flask(__name__)
client = None

logger = create_logger(__name__)

@app.route('/api/version/<string:version>')
def detail(version):
    client.reload_by_version(version)
    res = client.read()
    data = res
    return data

@app.route('/api/version/<string:version>/<string:type>')
def detail_type(version, type):
    client.reload_by_version(version)
    res = client.read()
    return res.get(type, {})

@app.route('/test', methods=['post', 'get'])
def test():
    res = {
        "args": request.args,
        "json": request.json,
        "data": str(request.data),
        "headers": dict(request.headers),
    }
    return res

PORT = 12000 + int(random_int(3, 1))
os.environ['WUSH_WEB_PORT'] = str(PORT)

def run_server(wapi, port=None):
    global client
    #  app.logger.disabled = True
    #  app.logger.setLevel("ERROR")
    #  log = logging.getLogger('werkzeug')
    #  logger.disabled = True
    client = wapi
    if not port:
        port = PORT
    app.run(host = '0.0.0.0', port=port)

if __name__ == "__main__":
    run_server()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

from .env import EnvArgumentParser
from .run import RunArgumentParser
from .func import FuncArgumentParser
from .view import ViewArgumentParser
from .config import ConfigArgumentParser
from .command import CmdArgumentParser

__all__ = [
    'EnvArgumentParser',
    'RunArgumentParser',
    'FuncArgumentParser',
    'ConfigArgumentParser',
    'CmdArgumentParser',
]



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
Wush
"""

import sys
import yaml
import os
import requests
import json
import cgi
import traceback

from datetime import datetime
from wush.common import constants
from wush.common import utils
from wush.common.files import FileUtils
from wush.common.loggers import create_logger
from wush.common.cookie import Cookie
from wush.common.config import Config
from wush.common.exceptions import RequestException

from wush.models import ModuleModel
from .models import Client
from .models import Version
from .models import Request
from .models import Response

class Wapi():
    logger = create_logger('Wush')
    module_name = ''
    request_name = ''
    space_name = ''
    config_root = ''
    _config = None
    _request = None
    _url = None
    _response = None
    is_dynamic_space = True

    def __init__(self, name=None, is_dynamic_space = True, **kw):
        if not name:
            name = __name__
        self.name = name
        self.is_dynamic_space = is_dynamic_space
        self.version = self.create_id()
        self._id = self.version
        self.config_root = Config.get_default_root()
        self.init_config(**kw)

    def reload_by_version(self, version):
        """通过版本号重新加载"""
        self.version = version
        #  data = FileUtils.read_dict(self.version_path)
        data = Version.find_one_by_id(version).to_dict()
        self.init_config(**data)

    @classmethod
    def create_id(cls):
        return datetime.now().strftime('%Y%m%d%H%M%S_%s')

    @property
    def config(self):
        """获取配置"""
        return self._config

    def _change_config(self, config_root=None, config=None):
        if self._config and not config_root and not config:
            return

        if config_root:
            self.config_root = config_root

        # 优先使用传入的 config
        if isinstance(config, dict):
            self._config = Config.load(config)

        # 在使用地址获取配置
        if not self._config and self.config_root:
            self._config = Config.load(self.config_root)

    def init_config(self,**kw):
        """初始化配置"""
        config = kw.pop('config', None)
        # config 地址
        config_root = kw.pop('config_root', None)

        # 改变配置
        self._change_config(config_root, config)

        self.logger.info('init_config kwargs %s', kw)
        for k, v in kw.items():
            if v:
                setattr(self, k, v)

        # 配置 space_name
        if not self.space_name or self.is_dynamic_space:
            self.logger.info('wush %s', self.name)
            current_space_name = self.config.get_function().get_current_space_name()
            self.logger.info('current_space_name %s', current_space_name)
            self.space_name = current_space_name

        self._config.set_space_name(self.space_name)
        self.logger.info("Space: %s", self.space_name)

        # 设置默认 module
        if not self.module_name:
            self.module_name = self.config.get_default_module_name()

        self._init_environ()
        self._build()

    def _init_environ(self):
        '''初始化环境变量'''
        current_body_name = self.config.get_current_body_name(
                self.space_name, self.module_name,self.request_name)
        current_body_path = self.config.get_body_path(current_body_name)
        self.config.env.add(**dict(
            body_path = current_body_path
        ))

    def build(self, **kwargs):
        """
        构建请求
        :param kwargs:
        """
        for k in ('space_name', 'module_name', 'request_name'):
            val = kwargs.pop(k, None)
            if val:
                setattr(self, k, val)

        self._build(**kwargs)
        return self

    def _build(self, **kwargs):
        """获取 request
        :param dict kwargs:
            `env` 环境变量
        """
        if not self.request_name or not self.module_name:
            return
        module = self.config.get_module(self.module_name)
        request = module.get_request(self.request_name, **kwargs)
        if not request:
            return
        self._request = request
        self.logger.info('RequestModel: %s', request)
        self._url = request.url
        self._request_data = { }
        # 注入 request 必要的参数
        for k in ('method', 'url'):
            self._request_data[k] = getattr(self._request, k)
        # 注入 request 必要的参数
        for name in ('json', 'data', 'headers', 'cookies', 'params'):
            value =  getattr(self._request, name)
            if value:
                self._request_data[name] = value
        self.logger.info('Request data: %s', self._request_data)

    def request(self, **kwargs):
        self.version = self.create_id()
        if kwargs:
            self.build(**kwargs)

        res = requests.request(**self._request_data)

        self.response_content = res.content
        self._response = res
        self.logger.info('Response')
        response_data = {}
        response_data['headers'] = dict(res.headers)
        self.logger.info(json.dumps(response_data, indent=4))
        return res

    @property
    def url(self):
        return self._url

    @property
    def response(self):
        return self._response

    def print_response(self):
        """打印返回结果"""
        print('Space:', self.space_name)
        print('Module:', self.module_name)
        print('Name:', self.request_name)
        print("Status:", self._response.status_code)
        try:
            data = self._response.json()
            try:
                data = utils.filter_json(data, self._request.filters)
            except Exception as e:
                pass
            print("Format:")
            print(json.dumps(data, indent=4, ensure_ascii=False))
        except:
            print('Content:')
            print(self._response.content)
            print('解析 json 失败')

    def get_pertty_response_content(self):
        """获取好看的 response content 内容"""
        try:
            data = self._response.json()
            try:
                data = utils.filter_json(data, self._request.filters)
            except:
                self.logger.error(traceback.format_exc())
            return json.dumps(data, indent=4, ensure_ascii=False)
        except:
            self.logger.error(traceback.format_exc())
            return self._response.content

    @property
    def response_path(self):
        """结果存储地址"""
        filename = '{}-{}'.format(self.module_name, self.request_name)
        if self._response:
            headers = dict(self._response.headers)
            content_type = headers.get("content-type") or ''
            # 处理office文件的报错
            if 'office' in content_type:
                content_disposition = headers.get("content-disposition")
                _, params = cgi.parse_header(content_disposition)
                filename = params.get("filename")

        path = os.path.join(self.config.response_root, '{}-{}'.format(
            self.version, filename))

        self.logger.info('response_path %s', path)
        return path

    def save(self):
        """保存请求配置"""
        self._save_version()
        self._save_request()
        res = Response(_id = self.version)
        try:
            res.json = json.loads(self.response_content)
        except Exception as e:
            print(e)
            res.content = self.response_content
        res.headers = dict(self.response.headers)
        res.save()

    def _save_version(self):
        data = {
            "version": self.version,
            "space_name": self.space_name,
            "module_name": self.module_name,
            "request_name": self.request_name,
            "config": self.config.dict(),
        }
        v = Version(_id = data['version'], **data)
        v.save()

    def _save_request(self):
        r = Request(_id = self.version, **self._request_data)
        r.save()

    def read(self):
        data = {
            "version": Version.find_one_by_id(self.version).to_dict(),
            "request": Request.find_one_by_id(self.version).to_dict(),
            "response": Response.find_one_by_id(self.version).to_dict(),
        }
        return data



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import abc
import sys
import os
import argparse
import shutil
import traceback
import pygments

from datetime import datetime
from pygments.token import Token
from pygments.lexers.python import PythonLexer
from prompt_toolkit.formatted_text import PygmentsTokens
from prompt_toolkit import print_formatted_text
from prompt_toolkit import PromptSession
from prompt_toolkit.application import run_in_terminal
from prompt_toolkit.history import FileHistory
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.completion import Completer, Completion
from prompt_toolkit.completion import WordCompleter

from wush.argument import ArgumentParser
from wush.argument import ArgumentParserFactory
from wush.argument import CommandArgumentParser
from wush.argument import EnvArgumentParser
from wush.common import utils
from wush.common.functions import super_function
from wush.common.functions import run_shell
from wush.common.files import FileUtils
from wush.common.loggers import create_logger
from wush.completion.command import CommandCompleter
from wush.wush import Wapi

from .exceptions import ContinueException
from .exceptions import CommnadNotFoundException

def init_argparse():
    """初始化参数"""
    parser = argparse.ArgumentParser(description='Wapi command',)
    parser.add_argument('-c', '--config', help='Config dir name')
    parser.add_argument('-m', '--module', help='Module name')
    parser.add_argument('-n', '--name', help='Request name')
    parser.add_argument('-s', '--space', help='Space name')
    return parser

class PromptToolkitShell(object, metaclass=abc.ABCMeta):
    logger = create_logger('Shell')

    parser_dict = {}
    parser = None
    _prompt_default = ''
    _prompt_right = ''
    _prompt_left = ''

    def __init__(self):
        self.parser = self._get_parser()
        args = init_argparse().parse_args()
        #  args = init_argparse().parse_args(sys.argv)
        client = Wapi()
        client.init_config(config_root = args.config, space_name = args.space,
            module_name = args.module)
        self.client = client
        self.session = PromptSession(
            completer=CommandCompleter(self.parser, client),
            history = FileHistory(os.path.expanduser('~/.wapi_history')),
            auto_suggest = AutoSuggestFromHistory(),
            complete_in_thread=True
        )

    def _get_parser(self, cmd=None):
        if cmd not in self.parser_dict:
            parser = ArgumentParserFactory.build_parser(cmd)
            if isinstance(parser, CommandArgumentParser):
                parser.set_wapi(self.client)
                parser.set_prompt_session(self.session)
            self.parser_dict[cmd] = parser
        return self.parser_dict[cmd]

    def run(self):
        self._before_run()
        self._run_shell()
        self._after_run()

    def _before_run(self):
        pass

    def _after_run(self):
        pass

    def set_prompt_left(self, text):
        self._prompt_left = text

    def set_prompt_right(self, text):
        self._prompt_right = text

    def set_prompt_default(self, text):
        self._prompt_default = text

    def _run_shell(self):
        while True:
            try:
                text = self.session.prompt(
                    self._prompt_left,
                    default = self._prompt_default,
                    rprompt = self._prompt_right,
                )
                self._run_once_time(text)
            except ContinueException:
                continue
            except CommnadNotFoundException:
                print('command not found: {}'.format(text))
            except KeyboardInterrupt:
                continue
            except EOFError:
                break
            except Exception as e:
                self._print('ERROR: ' + str(e))
                self.logger.error(traceback.format_exc())
            self._end_run()

    def _end_run(self):
        self._prompt_default = ''

    def _run_once_time(self, text):
        """运行"""
        if not text:
            return
        parser = self._get_parser()
        args = parser.parse_args(text)
        cmd = args.cmd
        self.parser = self._get_parser(cmd)
        self.logger.info('run argparser %s', self.parser)

        self._run_base_cmd(text)

        if isinstance(self.parser, CommandArgumentParser):
            self.parser.run(text)
            return

        if not hasattr(self, '_' + cmd):
            raise CommnadNotFoundException()

        func = getattr(self, '_' + cmd)
        func(text)

    def _run_base_cmd(self, text):
        """运行基础命令"""
        if text.startswith('!'):
            text = text[1:]
            try:
                history_num = int(text)
                self.logger.info(history_num)
                cmd = self.get_history_by_num(history_num)
                self._prompt_default = cmd
            except:
                self.logger.error(traceback.format_exc())
                raise CommnadNotFoundException()
            else:
                raise ContinueException()

    def _exit(self, text):
        raise EOFError()

    def get_history_by_num(self, num):
        """获取历史命令"""
        items = self.session.history.get_strings()
        if len(items) < num:
            return None
        return items[num - 1]

    def _print(self, text):
        tokens = list(pygments.lex(text, lexer=PythonLexer()))
        print_formatted_text(PygmentsTokens(tokens), end='')

    @abc.abstractmethod
    def _run_cmd(self, text):
        pass



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import logging

LOGFILE = '/tmp/wush.log'

class CustomerFilter(logging.Filter):

    def filter(self, record):
        if ':' not in record.filename:
            record.filename = '{}:{}'.format( record.filename, record.lineno)
        return True

logging.basicConfig(
    filename=LOGFILE,
    format="[%(asctime)s] [%(levelname)-5s] [%(filename)-20s] %(message)s",
    level=logging.DEBUG
)

def create_logger(name):
    logger = logging.getLogger(name)
    logger.addFilter(CustomerFilter())
    return logger



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""
枚举
"""
from enum import Enum


#  class Action(Enum):
    #  STORE = 'store'
    #  STORE_TRUE = 'store_true'
    #  APPEND = 'append'




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
#  from argparse import Namespace
#  from collections import deque

#  from wpy.argument import ArgumentParser as BaseParser
#  from wpy.argument import Argument
#  from wpy.argument import ArgumentNamespace

#  from wapi.common.loggers import create_logger
#  from .enum import Action

#  class ArgumentParser(BaseParser):
    #  logger = create_logger('ArgumentParser')
    #  cmd = ''

    #  @classmethod
    #  def default(cls):
        #  """
        #  初始化一个默认实例
        #  """
        #  item = cls()
        #  item.add_argument('cmd')
        #  item.add_argument('--config')
        #  item.add_argument('--module')
        #  item.add_argument('--space')
        #  return item

    #  def get_completions_after_argument(self, wapi, word_for_completion):
        #  """
        #  获取补全的单词列表
        #  :param wapi: Wapi
        #  :param word_for_completion: 补全需要的单词
        #  """
        #  return []

    #  def get_completions_after_cmd(self, argument, words=None):
        #  """获取补全使用的单词列表"""
        #  res = []
        #  if words and isinstance(words, list):
            #  res.extend(words)
        #  args = self.get_arguments()
        #  for arg in args:
            #  if arg.is_cmd:
                #  continue
            #  # 已经赋值的不需要展示
            #  if hasattr(argument, arg.name) and getattr(argument, arg.name):
                #  # 列表除外
                #  if not arg.is_list:
                    #  continue
            #  res.append(dict(text = '--' + arg.name, display_meta = arg.help))
        #  return res



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
import sys
import os
import argparse
import shutil
import traceback

from wush.argument import EnvArgumentParser
from wush.common.functions import super_function
from wush.common.loggers import create_logger
from wush.completion.command import CommandCompleter
from wush.wush import Wapi

from .shell import Shell

logger = create_logger('main')

def run(client):
    res = client.request()
    client.print_response()
    client.save()

def body(args):

    config = client.config
    default_body_name = config.get_current_body_name('default', client.module_name, client.request_name)
    default_body_path = config.get_body_path(default_body_name)
    body_name = config.get_current_body_name(client.space_name, client.module_name, client.request_name)
    body_path = config.get_body_path(body_name)
    logger.info('Default body path: %s', default_body_path)
    logger.info('Body path: %s', body_path)
    # 如果存在默认 body 文件，切不存在 space body 文件时，copy 默认文件
    if os.path.exists(default_body_path) and not os.path.exists(body_path):
        shutil.copy(default_body_path, body_path)
    os.system('vim {}'.format(body_path))
    # 如果不存在默认文件，生成一份
    if not os.path.exists(default_body_path):
        shutil.copy(body_path, default_body_path)

def env(client):
    client.config.env.add
    config = client.config
    name = config.get_current_env_name(client.space_name)
    path = config.get_env_path(name)
    os.system('vim {}'.format(path))

def module(args):
    config = client.config
    path = config.get_module_path(client.module_name)
    os.system('vim {}'.format(path))

func_dict = {
    "run": run,
    "body": body,
    "env": env,
    "module": module,
}

def init_argparse():
    """初始化参数"""
    parser = argparse.ArgumentParser(description='Wush command',)
    parser.add_argument('cmd', help='You can use run, body, env, module')
    parser.add_argument('-c', '--config', help='Config dir name')
    parser.add_argument('-m', '--module', help='Module name')
    parser.add_argument('-n', '--name', help='Request name')
    parser.add_argument('-s', '--space', help='Space name')
    return parser

def run_cmd():
    parser = init_argparse()
    args = parser.parse_args()
    cmd = args.cmd
    name = args.name
    if cmd == 'run':
        if not name:
            raise Exception

    client = Wapi()
    client.init_config(
        space_name = args.space,
        module_name = args.module,
        request_name = args.name,
        config_root = args.config)
    func_dict.get(cmd)(client)

from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory

def run_shell():

    cli = Shell()
    cli.run()

def main():
    import sys
    args = sys.argv[1:]
    run_shell()

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""

import os
import json
import yaml

class FileUtils:

    @classmethod
    def read_dict(cls, filepath):
        """
        读取字典数据
        :param str filepath: 文件地址
        """
        with open(filepath, 'r') as f:
            if filepath.endswith('.yml'):
                return yaml.safe_load(f)
            lines = f.readlines()
        return json.loads(''.join(lines))

    @classmethod
    def save_yml(cls, filepath, data):
        """保存成 yml 格式文件"""
        filepath = os.path.expanduser(filepath)
        with open(filepath, 'w') as f:
            yaml.dump(data, f)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy@gmail.com
"""

"""
from collections import deque

from wpy.files import FileUtils
from wush.common.loggers import create_logger
from .command import CmdArgumentParser
from wpy.argument import CommandArgumentParserFactory

@CommandArgumentParserFactory.register()
class EnvArgumentParser(CmdArgumentParser):
    cmd = 'env'

    def get_completions_after_cmd(self, argument, words=None):
        words = []
        for k ,v in self.wapi.config.env.dict().items():
            text = '--{}'.format(k)
            display = '--{}={}'.format(k, v)
            words.append(dict(text = text, display = display))
        return super().get_completions_after_cmd(argument, words)

    def _make_args_dict(self, args):
        res = super()._make_args_dict(args)
        args = list(filter(lambda x: x != '--save', args))
        if len(args) < 3:
            return res
        # 将所有成对的参数都放入参数列表
        d_args = deque(args[1:])
        d_args.rotate(-1)
        for i, (k, v) in enumerate(zip(args[1:], d_args)):
            if i % 2  == 1 or not k.startswith('--'):
                continue
            k = k.replace('--', '')
            res[k] = v
        return res

    @classmethod
    def default(cls):
        """
        初始化一个实例
        """
        item = cls()
        item.add_argument('cmd')
        item.add_argument('--save', action='store_true')
        return item

    def run(self, args):
        """执行"""
        args = self.parse_args(args)
        if args.has_args():
            arg_names = [o.name for o in self.get_arguments()]
            for k, v in args.__dict__.items():
                if k not in arg_names:
                    self.wapi.config.env.add(**{ k: v })
                    print('{}={}'.format(k, v))
            self.wapi.init_config()
        else:
            for k, v in self.wapi.config.env.dict().items():
                print('{}={}'.format(k, v))

        # 保存数据
        if args.save:
            env_path = self.wapi.config.get_env_path()
            env_data = {}
            try:
                env_data = FileUtils.read_dict(env_path)
            except:
                pass
            for k, v in self.wapi.config.env.dict().items():
                if k in ('body_path',):
                    continue
                if k.isupper():
                    continue
                if not v:
                    continue
                if env_data.get(k) == v:
                    continue
                self.logger.info('Env save %s=%s', k, v)
                env_data[k] = v
            FileUtils.write_yml(env_path, env_data)

